@@ -9,7 +9,7 @@
 
 #include <sys/stat.h>
 
-int cow_cp(const char *path, int branch_ro, int branch_rw);
+int cow_cp(const char *path, int branch_ro, int branch_rw, bool copy_dir);
 int path_create(const char *path, int nbranch_ro, int nbranch_rw);
 int path_create_cutlast(const char *path, int nbranch_ro, int nbranch_rw);
 int copy_directory(const char *path, int branch_ro, int branch_rw);
