@@ -20,13 +20,6 @@
 #endif
 
 #include <fuse.h>
-
-#if __APPLE__
-#include <fuse_common.h>
-#else
-#include <fuse/fuse_common.h>
-#endif
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -39,6 +32,7 @@
 #include <sys/types.h>
 #include <sys/time.h>
 #include <inttypes.h>
+#include <sys/ioctl.h>
 
 #ifdef linux
 	#include <sys/vfs.h>
@@ -65,6 +59,14 @@
 #include "conf.h"
 #include "uioctl.h"
 
+#ifndef _IOC_SIZE
+#ifdef IOCPARM_LEN
+#define _IOC_SIZE(x) IOCPARM_LEN(x)
+#else
+#error "No mechanism for determining ioctl length found."
+#endif
+#endif
+
 static struct fuse_opt unionfs_opts[] = {
 	FUSE_OPT_KEY("chroot=%s,", KEY_CHROOT),
 	FUSE_OPT_KEY("cow", KEY_COW),
@@ -236,7 +238,7 @@ static void * unionfs_init(struct fuse_conn_info *conn) {
 
 #ifdef FUSE_CAP_IOCTL_DIR
 	if (conn->capable & FUSE_CAP_IOCTL_DIR)
-		conn->want |= FUSE_CAP_IOCTL_DIR; 
+		conn->want |= FUSE_CAP_IOCTL_DIR;
 #endif
 
 	return NULL;
@@ -267,6 +269,7 @@ static int unionfs_link(const char *from, const char *to) {
 	RETURN(0);
 }
 
+#if FUSE_VERSION >= 28
 static int unionfs_ioctl(const char *path, int cmd, void *arg, struct fuse_file_info *fi, unsigned int flags, void *data) {
 	(void) path;
 	(void) arg; // avoid compiler warning
@@ -275,8 +278,10 @@ static int unionfs_ioctl(const char *path, int cmd, void *arg, struct fuse_file_
 
 	fprintf(stderr, "Got ioctl: %d\n", cmd);
 
+#ifdef FUSE_IOCTL_COMPAT // 32bit-mode within 64-bit
 	if (flags & FUSE_IOCTL_COMPAT)
 		return -ENOSYS;
+#endif
 
 	switch (cmd) {
 	case UNIONFS_ONOFF_DEBUG: {
@@ -303,6 +308,7 @@ static int unionfs_ioctl(const char *path, int cmd, void *arg, struct fuse_file_
 
 	return 0;
 }
+#endif
 
 /**
  * unionfs mkdir() implementation
@@ -453,7 +459,7 @@ static int unionfs_rename(const char *from, const char *to) {
 	if (i == -1) RETURN(-errno);
 
 	if (!uopt.branches[i].rw) {
-		i = find_rw_branch_cow(from);
+		i = find_rw_branch_cow_common(from, true);
 		if (i == -1) RETURN(-errno);
 	}
 
@@ -621,7 +627,7 @@ static int unionfs_statfs(const char *path, struct statvfs *stbuf) {
 				stbuf->f_favail += stb.f_favail;
 			} else if (!uopt.statfs_omit_ro) {
 				// omitting the RO branches is not correct regarding
-				// the block counts but it actually fixes the 
+				// the block counts but it actually fixes the
 				// percentage of free space. so, let the user decide.
 				stbuf->f_blocks += stb.f_blocks * ratio;
 				stbuf->f_files  += stb.f_files;
@@ -685,9 +691,9 @@ static int unionfs_utimens(const char *path, const struct timespec ts[2]) {
 #else
 	struct timeval tv[2];
 	tv[0].tv_sec = ts[0].tv_sec;
-	tv[0].tv_usec = ts[0].tv_nsec * 1000;
+	tv[0].tv_usec = ts[0].tv_nsec / 1000;
 	tv[1].tv_sec = ts[1].tv_sec;
-	tv[1].tv_usec = ts[1].tv_nsec * 1000;
+	tv[1].tv_usec = ts[1].tv_nsec / 1000;
 	int res = utimes(p, tv);
 #endif
 
@@ -806,7 +812,9 @@ static struct fuse_operations unionfs_oper = {
 	.fsync = unionfs_fsync,
 	.getattr = unionfs_getattr,
 	.init = unionfs_init,
+#if FUSE_VERSION >= 28
 	.ioctl = unionfs_ioctl,
+#endif
 	.link = unionfs_link,
 	.mkdir = unionfs_mkdir,
 	.mknod = unionfs_mknod,
@@ -850,10 +858,10 @@ int main(int argc, char *argv[]) {
 
 	// enable fuse permission checks, we need to set this, even we we are
 	// not root, since we don't have our own access() function
-	int uid = getuid();
-	int gid = getgid();
+	uid_t uid = getuid();
+	gid_t gid = getgid();
 	bool default_permissions = true;
-	
+
 	if (uid != 0 && gid != 0 && uopt.relaxed_permissions) {
 		default_permissions = false;
 	} else if (uopt.relaxed_permissions) {
@@ -861,7 +869,7 @@ int main(int argc, char *argv[]) {
 		fprintf(stderr, "Relaxed permissions disallowed for root!\n");
 		exit(1);
 	}
-	
+
 	if (default_permissions) {
 		if (fuse_opt_add_arg(&args, "-odefault_permissions")) {
 			fprintf(stderr, "Severe failure, can't enable permssion checks, aborting!\n");
@@ -871,7 +879,7 @@ int main(int argc, char *argv[]) {
 	unionfs_post_opts();
 
 #ifdef FUSE_CAP_BIG_WRITES
-	/* libfuse > 0.8 supports large IO, also for reads, to increase performance 
+	/* libfuse > 0.8 supports large IO, also for reads, to increase performance
 	 * We support any IO sizes, so lets enable that option */
 	if (fuse_opt_add_arg(&args, "-obig_writes")) {
 		fprintf(stderr, "Failed to enable big writes!\n");
